@@ -435,7 +435,8 @@ asb_utils_write_archive (const gchar *filename,
 
 		tmp = g_ptr_array_index (files, i);
 		filename_full = g_build_filename (path_orig, tmp, NULL);
-		stat (filename_full, &st);
+		if (stat (filename_full, &st) != 0)
+			continue;
 		entry = archive_entry_new ();
 		archive_entry_set_pathname (entry, tmp);
 		archive_entry_set_size (entry, st.st_size);
@@ -443,12 +444,13 @@ asb_utils_write_archive (const gchar *filename,
 		archive_entry_set_perm (entry, 0644);
 		archive_write_header (a, entry);
 		ret = g_file_get_contents (filename_full, &data, &len, error);
-		if (!ret)
-			goto out;
+		if (!ret) {
+			archive_entry_free (entry);
+			break;
+		}
 		archive_write_data (a, data, len);
 		archive_entry_free (entry);
 	}
-out:
 	archive_write_close (a);
 	archive_write_free (a);
 	return ret;
