@@ -68,18 +68,20 @@ struct _AsAppClass
  * @AS_APP_PARSE_FLAG_ALLOW_VETO:	Do not return errors for vetoed apps
  * @AS_APP_PARSE_FLAG_USE_FALLBACKS:	Fall back to suboptimal data where required
  * @AS_APP_PARSE_FLAG_ADD_ALL_METADATA:	Add all extra metadata from the source file
+ * @AS_APP_PARSE_FLAG_ONLY_NATIVE_LANGS:	Only load native languages
  *
  * The flags to use when parsing resources.
  **/
 typedef enum {
-	AS_APP_PARSE_FLAG_NONE,
-	AS_APP_PARSE_FLAG_USE_HEURISTICS	= 1,	/* Since: 0.1.2 */
-	AS_APP_PARSE_FLAG_KEEP_COMMENTS		= 2,	/* Since: 0.1.6 */
-	AS_APP_PARSE_FLAG_CONVERT_TRANSLATABLE	= 4,	/* Since: 0.1.6 */
-	AS_APP_PARSE_FLAG_APPEND_DATA		= 8,	/* Since: 0.1.8 */
-	AS_APP_PARSE_FLAG_ALLOW_VETO		= 16,	/* Since: 0.2.5 */
-	AS_APP_PARSE_FLAG_USE_FALLBACKS		= 32,	/* Since: 0.4.1 */
-	AS_APP_PARSE_FLAG_ADD_ALL_METADATA	= 64,	/* Since: 0.6.1 */
+	AS_APP_PARSE_FLAG_NONE			= 0,
+	AS_APP_PARSE_FLAG_USE_HEURISTICS	= 1 << 0,	/* Since: 0.1.2 */
+	AS_APP_PARSE_FLAG_KEEP_COMMENTS		= 1 << 1,	/* Since: 0.1.6 */
+	AS_APP_PARSE_FLAG_CONVERT_TRANSLATABLE	= 1 << 2,	/* Since: 0.1.6 */
+	AS_APP_PARSE_FLAG_APPEND_DATA		= 1 << 3,	/* Since: 0.1.8 */
+	AS_APP_PARSE_FLAG_ALLOW_VETO		= 1 << 4,	/* Since: 0.2.5 */
+	AS_APP_PARSE_FLAG_USE_FALLBACKS		= 1 << 5,	/* Since: 0.4.1 */
+	AS_APP_PARSE_FLAG_ADD_ALL_METADATA	= 1 << 6,	/* Since: 0.6.1 */
+	AS_APP_PARSE_FLAG_ONLY_NATIVE_LANGS	= 1 << 7,	/* Since: 0.6.3 */
 	/*< private >*/
 	AS_APP_PARSE_FLAG_LAST,
 } AsAppParseFlags;
@@ -180,6 +182,7 @@ typedef enum {
 					 AS_APP_SUBSUME_FLAG_KUDOS | \
 					 AS_APP_SUBSUME_FLAG_LANGUAGES | \
 					 AS_APP_SUBSUME_FLAG_MIMETYPES | \
+					 AS_APP_SUBSUME_FLAG_METADATA | \
 					 AS_APP_SUBSUME_FLAG_NAME | \
 					 AS_APP_SUBSUME_FLAG_PERMISSIONS | \
 					 AS_APP_SUBSUME_FLAG_PROJECT_GROUP | \
@@ -334,6 +337,7 @@ typedef enum {
  * @AS_APP_QUIRK_HAS_SHORTCUT:		The app has a shortcut in the system
  * @AS_APP_QUIRK_NOT_LAUNCHABLE:	The app is not launchable (run-able)
  * @AS_APP_QUIRK_NEEDS_USER_ACTION:	The component requires some kind of user action
+ * @AS_APP_QUIRK_IS_PROXY:		Is a proxy app that operates on other applications
  *
  * The component attributes.
  **/
@@ -348,6 +352,7 @@ typedef enum {
 	AS_APP_QUIRK_HAS_SHORTCUT	= 1 << 6,	/* Since: 0.5.15 */
 	AS_APP_QUIRK_NOT_LAUNCHABLE	= 1 << 7,	/* Since: 0.5.15 */
 	AS_APP_QUIRK_NEEDS_USER_ACTION	= 1 << 8,	/* Since: 0.6.2 */
+	AS_APP_QUIRK_IS_PROXY 		= 1 << 9,	/* Since: 0.6.6 */
 	/*< private >*/
 	AS_APP_QUIRK_LAST
 } AsAppQuirk;
@@ -420,6 +425,33 @@ typedef enum {
 	AS_APP_MERGE_KIND_LAST
 } AsAppMergeKind;
 
+/**
+ * AsAppSearchMatch:
+ * @AS_APP_SEARCH_MATCH_NONE:			No token matching
+ * @AS_APP_SEARCH_MATCH_MIMETYPE:		Use the app mimetypes
+ * @AS_APP_SEARCH_MATCH_PKGNAME:		Use the app package name
+ * @AS_APP_SEARCH_MATCH_DESCRIPTION:		Use the app description
+ * @AS_APP_SEARCH_MATCH_COMMENT:		Use the app comment
+ * @AS_APP_SEARCH_MATCH_NAME:			Use the app name
+ * @AS_APP_SEARCH_MATCH_KEYWORD:		Use the app keyword
+ * @AS_APP_SEARCH_MATCH_ID:			Use the app application ID
+ *
+ * The token match kind, which we want to be exactly 16 bits for storage
+ * reasons.
+ **/
+typedef enum __attribute__((__packed__)) {
+	AS_APP_SEARCH_MATCH_NONE	= 0,		/* Since: 0.6.5 */
+	AS_APP_SEARCH_MATCH_MIMETYPE	= 1 << 0,	/* Since: 0.6.5 */
+	AS_APP_SEARCH_MATCH_PKGNAME	= 1 << 1,	/* Since: 0.6.5 */
+	AS_APP_SEARCH_MATCH_DESCRIPTION	= 1 << 2,	/* Since: 0.6.5 */
+	AS_APP_SEARCH_MATCH_COMMENT	= 1 << 3,	/* Since: 0.6.5 */
+	AS_APP_SEARCH_MATCH_NAME	= 1 << 4,	/* Since: 0.6.5 */
+	AS_APP_SEARCH_MATCH_KEYWORD	= 1 << 5,	/* Since: 0.6.5 */
+	AS_APP_SEARCH_MATCH_ID		= 1 << 6,	/* Since: 0.6.5 */
+	/*< private >*/
+	AS_APP_SEARCH_MATCH_LAST	= 0xffff
+} AsAppSearchMatch;
+
 #define	AS_APP_ERROR				as_app_error_quark ()
 
 AsApp		*as_app_new			(void);
