@@ -39,20 +39,38 @@
 
 #include "as-markup.h"
 #include "as-node-private.h"
+#include "as-ref-string.h"
 #include "as-utils-private.h"
 
+typedef struct {
+	GHashTable		*intern_attr;	/* key=value of AsRefString */
+	GHashTable		*intern_name;	/* key=value of AsRefString */
+} AsNodeRoot;
+
 typedef struct
 {
-	GList		*attrs;
-	gchar		*name;		/* only used if tag == AS_TAG_UNKNOWN */
-	gchar		*cdata;
-	gboolean	 cdata_escaped;
-	AsTag		 tag;
+	GList			*attrs;
+	union {
+		AsTag		 tag;
+		const gchar	*name_const;	/* only if is_name_const = TRUE */
+		AsRefString	*name;		/* only if is_tag_valid = FALSE */
+	};
+	union {
+		AsNodeRoot	*root;		/* only if is_root_node = TRUE */
+		const gchar	*cdata_const;	/* only if is_cdata_const = TRUE */
+		AsRefString	*cdata;
+	};
+	guint8			 is_root_node:1;
+	guint8			 is_cdata_const:1;
+	guint8			 is_name_const:1;
+	guint8			 is_cdata_escaped:1;
+	guint8			 is_cdata_ignore:1;
+	guint8			 is_tag_valid:1;
 } AsNodeData;
 
 typedef struct {
-	const gchar	*key;
-	gchar		*value;
+	AsRefString		*key;
+	AsRefString		*value;
 } AsNodeAttr;
 
 /**
@@ -70,23 +88,49 @@ as_node_new (void)
 	AsNodeData *data;
 	data = g_slice_new0 (AsNodeData);
 	data->tag = AS_TAG_LAST;
+	data->is_tag_valid = TRUE;
+	data->is_root_node = TRUE;
+	data->root = g_new0 (AsNodeRoot, 1);
+	data->root->intern_attr = g_hash_table_new_full (g_str_hash,
+							 g_str_equal,
+							 (GDestroyNotify) as_ref_string_unref,
+							 NULL);
+	data->root->intern_name = g_hash_table_new_full (g_str_hash,
+							 g_str_equal,
+							 (GDestroyNotify) as_ref_string_unref,
+							 NULL);
 	return g_node_new (data);
 }
 
 static void
 as_node_attr_free (AsNodeAttr *attr)
 {
-	g_free (attr->value);
 	g_slice_free (AsNodeAttr, attr);
 }
 
+static AsRefString *
+as_node_intern (GHashTable *hash, const gchar *key)
+{
+	AsRefString *rstr = g_hash_table_lookup (hash, key);
+	if (rstr == NULL) {
+		rstr = as_ref_string_new (key);
+		g_hash_table_insert (hash, rstr, rstr);
+	}
+	return rstr;
+}
+
 static AsNodeAttr *
-as_node_attr_insert (AsNodeData *data, const gchar *key, const gchar *value)
+as_node_attr_insert (AsNode *root,
+		     AsNodeData *data,
+		     const gchar *key,
+		     const gchar *value)
 {
 	AsNodeAttr *attr;
+	AsNodeRoot *root_data = ((AsNodeData *)root->data)->root;
+
 	attr = g_slice_new0 (AsNodeAttr);
-	attr->key = g_intern_string (key);
-	attr->value = g_strdup (value);
+	attr->key = as_node_intern (root_data->intern_attr, key);
+	attr->value = as_node_intern (root_data->intern_attr, value);
 	data->attrs = g_list_prepend (data->attrs, attr);
 	return attr;
 }
@@ -121,8 +165,16 @@ as_node_destroy_node_cb (AsNode *node, gpointer user_data)
 	AsNodeData *data = node->data;
 	if (data == NULL)
 		return FALSE;
-	g_free (data->name);
-	g_free (data->cdata);
+	if (!data->is_tag_valid && !data->is_name_const && data->name != NULL)
+		as_ref_string_unref (data->name);
+	if (data->is_root_node) {
+		g_hash_table_unref (data->root->intern_attr);
+		g_hash_table_unref (data->root->intern_name);
+		g_free (data->root);
+	} else {
+		if (!data->is_cdata_const && data->cdata != NULL)
+			as_ref_string_unref (data->cdata);
+	}
 	g_list_free_full (data->attrs, (GDestroyNotify) as_node_attr_free);
 	g_slice_free (AsNodeData, data);
 	return FALSE;
@@ -179,30 +231,61 @@ as_node_string_replace_inplace (gchar *text,
 	}
 }
 
+static void
+as_node_cdata_to_heap (AsNodeData *data)
+{
+	if (!data->is_cdata_const)
+		return;
+	data->cdata = as_ref_string_new (data->cdata);
+	data->is_cdata_const = FALSE;
+}
+
+static void
+as_node_cdata_to_intern (AsNode *root, AsNodeData *data)
+{
+	AsNodeRoot *root_data = ((AsNodeData *)root->data)->root;
+	AsRefString *tmp;
+	if (data->is_cdata_const)
+		return;
+	tmp = as_node_intern (root_data->intern_attr, data->cdata);
+	as_ref_string_unref (data->cdata);
+	data->cdata_const = tmp;
+	data->is_cdata_const = TRUE;
+}
+
 static void
 as_node_cdata_to_raw (AsNodeData *data)
 {
-	if (!data->cdata_escaped)
+	if (data->is_root_node)
 		return;
+	if (!data->is_cdata_escaped)
+		return;
+	if (data->is_cdata_const)
+		as_node_cdata_to_heap (data);
 	as_node_string_replace_inplace (data->cdata, "&amp;", '&');
 	as_node_string_replace_inplace (data->cdata, "&lt;", '<');
 	as_node_string_replace_inplace (data->cdata, "&gt;", '>');
-	data->cdata_escaped = FALSE;
+	data->is_cdata_escaped = FALSE;
 }
 
 static void
 as_node_cdata_to_escaped (AsNodeData *data)
 {
-	GString *str;
-	if (data->cdata_escaped)
+	if (data->is_root_node)
 		return;
-	str = g_string_new (data->cdata);
-	g_free (data->cdata);
-	as_utils_string_replace (str, "&", "&amp;");
-	as_utils_string_replace (str, "<", "&lt;");
-	as_utils_string_replace (str, ">", "&gt;");
-	data->cdata = g_string_free (str, FALSE);
-	data->cdata_escaped = TRUE;
+	if (data->is_cdata_escaped)
+		return;
+	if (g_strstr_len (data->cdata, -1, "&") != NULL ||
+	    g_strstr_len (data->cdata, -1, "<") != NULL ||
+	    g_strstr_len (data->cdata, -1, ">") != NULL) {
+		g_autoptr(GString) str = g_string_new (data->cdata);
+		as_ref_string_unref (data->cdata);
+		as_utils_string_replace (str, "&", "&amp;");
+		as_utils_string_replace (str, "<", "&lt;");
+		as_utils_string_replace (str, ">", "&gt;");
+		data->cdata = as_ref_string_new_copy_with_length (str->str, str->len);
+	}
+	data->is_cdata_escaped = TRUE;
 }
 
 static void
@@ -253,22 +336,35 @@ as_node_get_attr_string (AsNodeData *data)
 static const gchar *
 as_tag_data_get_name (AsNodeData *data)
 {
-	if (data->name == NULL)
+	if (data->is_tag_valid)
 		return as_tag_to_string (data->tag);
 	return data->name;
 }
 
 static void
-as_node_data_set_name (AsNodeData *data, const gchar *name, AsNodeInsertFlags flags)
+as_node_data_set_name (AsNode *root,
+		       AsNodeData *data,
+		       const gchar *name,
+		       AsNodeInsertFlags flags)
 {
 	if ((flags & AS_NODE_INSERT_FLAG_MARK_TRANSLATABLE) == 0) {
 		/* only store the name if the tag is not recognised */
-		data->tag = as_tag_from_string (name);
-		if (data->tag == AS_TAG_UNKNOWN)
-			data->name = g_strdup (name);
+		AsTag tag = as_tag_from_string (name);
+		if (tag == AS_TAG_UNKNOWN) {
+			AsNodeRoot *root_data = ((AsNodeData *)root->data)->root;
+			data->name_const = as_node_intern (root_data->intern_name, name);
+			data->is_name_const = TRUE;
+			data->is_tag_valid = FALSE;
+		} else {
+			data->name = NULL;
+			data->tag = tag;
+			data->is_tag_valid = TRUE;
+		}
 	} else {
 		/* always store the translated tag */
-		data->name = g_strdup_printf ("_%s", name);
+		g_autofree gchar *name_tmp = g_strdup_printf ("_%s", name);
+		data->name = as_ref_string_new (name_tmp);
+		data->is_tag_valid = FALSE;
 	}
 }
 
@@ -344,8 +440,10 @@ as_node_to_xml_string (GString *xml,
 	} else if (n->children == NULL) {
 		if ((flags & AS_NODE_TO_XML_FLAG_FORMAT_INDENT) > 0)
 			as_node_add_padding (xml, depth - depth_offset);
-		attrs = as_node_get_attr_string (data);
 		tag_str = as_tag_data_get_name (data);
+		if (tag_str == NULL)
+			return;
+		attrs = as_node_get_attr_string (data);
 		if (data->cdata == NULL || data->cdata[0] == '\0') {
 			g_string_append_printf (xml, "<%s%s/>",
 						tag_str, attrs);
@@ -397,13 +495,22 @@ as_node_to_xml_string (GString *xml,
  *
  * Since: 0.1.4
  **/
-gchar *
+AsRefString *
 as_node_reflow_text (const gchar *text, gssize text_len)
 {
-	GString *tmp;
 	guint i;
 	guint newline_count = 0;
 	g_auto(GStrv) split = NULL;
+	g_autoptr(GString) tmp = NULL;
+
+	/* all on one line, no trailing or leading whitespace */
+	if (g_strstr_len (text, text_len, "\n") == NULL &&
+	    !g_str_has_prefix (text, " ") &&
+	    !g_str_has_suffix (text, " ")) {
+		gsize len;
+		len = text_len >= 0 ? (gsize) text_len : strlen (text);
+		return as_ref_string_new_copy_with_length (text, len);
+	}
 
 	/* split the text into lines */
 	tmp = g_string_sized_new ((gsize) text_len + 1);
@@ -437,12 +544,13 @@ as_node_reflow_text (const gchar *text, gssize text_len)
 		/* this last section was paragraph */
 		newline_count = 1;
 	}
-	return g_string_free (tmp, FALSE);
+	return as_ref_string_new_copy_with_length (tmp->str, tmp->len);
 }
 
 typedef struct {
 	AsNode			*current;
 	AsNodeFromXmlFlags	 flags;
+	const gchar * const	*locales;
 } AsNodeToXmlHelper;
 
 /**
@@ -488,27 +596,53 @@ as_node_start_element_cb (GMarkupParseContext *context,
 {
 	AsNodeToXmlHelper *helper = (AsNodeToXmlHelper *) user_data;
 	AsNodeData *data;
+	AsNodeData *data_parent;
 	AsNode *current;
-	gchar *tmp;
+	AsRefString *tmp;
 	guint i;
 
-	/* create the new node data */
+	/* check if we should ignore the locale */
 	data = g_slice_new0 (AsNodeData);
-	as_node_data_set_name (data, element_name, AS_NODE_INSERT_FLAG_NONE);
-	for (i = 0; attribute_names[i] != NULL; i++) {
-		as_node_attr_insert (data,
-				     attribute_names[i],
-				     attribute_values[i]);
+
+	/* parent node is being ignored */
+	data_parent = helper->current->data;
+	if (data_parent->is_cdata_ignore)
+		data->is_cdata_ignore = TRUE;
+
+	/* check if we should ignore the locale */
+	if (!data->is_cdata_ignore &&
+	    helper->flags & AS_NODE_FROM_XML_FLAG_ONLY_NATIVE_LANGS) {
+		for (i = 0; attribute_names[i] != NULL; i++) {
+			if (g_strcmp0 (attribute_names[i], "xml:lang") == 0) {
+				const gchar *lang = attribute_values[i];
+				if (lang != NULL && !g_strv_contains (helper->locales, lang))
+					data->is_cdata_ignore = TRUE;
+			}
+		}
+	}
+
+	/* create the new node data */
+	if (!data->is_cdata_ignore) {
+		AsNode *root = g_node_get_root (helper->current);
+		as_node_data_set_name (root,
+				       data,
+				       element_name,
+				       AS_NODE_INSERT_FLAG_NONE);
+		for (i = 0; attribute_names[i] != NULL; i++) {
+			as_node_attr_insert (root, data,
+					     attribute_names[i],
+					     attribute_values[i]);
+		}
 	}
 
 	/* add the node to the DOM */
 	current = g_node_append_data (helper->current, data);
 
 	/* transfer the ownership of the comment to the new child */
-	tmp = as_node_take_attribute (helper->current, "@comment-tmp");
+	tmp = as_node_get_attribute (helper->current, "@comment-tmp");
 	if (tmp != NULL) {
 		as_node_add_attribute (current, "@comment", tmp);
-		g_free (tmp);
+		as_node_remove_attribute (helper->current, "@comment-tmp");
 	}
 
 	/* the child is now the node to be processed */
@@ -540,6 +674,11 @@ as_node_text_cb (GMarkupParseContext *context,
 	if (text_len == 0)
 		return;
 
+	/* ignoring */
+	data = helper->current->data;
+	if (data->is_cdata_ignore)
+		return;
+
 	/* all whitespace? */
 	for (i = 0; i < text_len; i++) {
 		if (!g_ascii_isspace(text[i]))
@@ -549,7 +688,6 @@ as_node_text_cb (GMarkupParseContext *context,
 		return;
 
 	/* split up into lines and add each with spaces stripped */
-	data = helper->current->data;
 	if (data->cdata != NULL) {
 		g_set_error (error,
 			     AS_NODE_ERROR,
@@ -560,10 +698,36 @@ as_node_text_cb (GMarkupParseContext *context,
 		return;
 	}
 	if ((helper->flags & AS_NODE_FROM_XML_FLAG_LITERAL_TEXT) > 0) {
-		data->cdata = g_strndup (text, text_len);
+		data->cdata = as_ref_string_new_with_length (text, text_len + 1);
 	} else {
 		data->cdata = as_node_reflow_text (text, (gssize) text_len);
 	}
+
+	/* intern commonly duplicated tag values and save a bit of memory */
+	if (data->is_tag_valid && data->cdata != NULL) {
+		AsNode *root = g_node_get_root (helper->current);
+		switch (data->tag) {
+		case AS_TAG_CATEGORY:
+		case AS_TAG_COMPULSORY_FOR_DESKTOP:
+		case AS_TAG_CONTENT_ATTRIBUTE:
+		case AS_TAG_DEVELOPER_NAME:
+		case AS_TAG_EXTENDS:
+		case AS_TAG_ICON:
+		case AS_TAG_ID:
+		case AS_TAG_KUDO:
+		case AS_TAG_LANG:
+		case AS_TAG_METADATA_LICENSE:
+		case AS_TAG_MIMETYPE:
+		case AS_TAG_PROJECT_GROUP:
+		case AS_TAG_PROJECT_LICENSE:
+		case AS_TAG_SOURCE_PKGNAME:
+		case AS_TAG_URL:
+			as_node_cdata_to_intern (root, data);
+			break;
+		default:
+			break;
+		}
+	}
 }
 
 static void
@@ -649,6 +813,7 @@ as_node_from_xml (const gchar *data,
 	root = as_node_new ();
 	helper.flags = flags;
 	helper.current = root;
+	helper.locales = g_get_language_names ();
 	ctx = g_markup_parse_context_new (&parser,
 					  G_MARKUP_PREFIX_ERROR_POSITION,
 					  &helper,
@@ -784,6 +949,7 @@ as_node_from_file (GFile *file,
 	root = as_node_new ();
 	helper.flags = flags;
 	helper.current = root;
+	helper.locales = g_get_language_names ();
 	ctx = g_markup_parse_context_new (&parser,
 					  G_MARKUP_PREFIX_ERROR_POSITION,
 					  &helper,
@@ -889,6 +1055,7 @@ void
 as_node_set_name (AsNode *node, const gchar *name)
 {
 	AsNodeData *data;
+	AsNode *root = g_node_get_root (node);
 
 	g_return_if_fail (node != NULL);
 
@@ -899,9 +1066,12 @@ as_node_set_name (AsNode *node, const gchar *name)
 		return;
 
 	/* overwrite */
-	g_free (data->name);
-	data->name = NULL;
-	as_node_data_set_name (data, name, AS_NODE_INSERT_FLAG_NONE);
+	if (!data->is_tag_valid) {
+		if (!data->is_name_const)
+			as_ref_string_unref (data->name);
+		data->name = NULL;
+	}
+	as_node_data_set_name (root, data, name, AS_NODE_INSERT_FLAG_NONE);
 }
 
 /**
@@ -922,6 +1092,8 @@ as_node_get_data (const AsNode *node)
 	if (node->data == NULL)
 		return NULL;
 	data = (AsNodeData *) node->data;
+	if (data->is_root_node)
+		return NULL;
 	if (data->cdata == NULL || data->cdata[0] == '\0')
 		return NULL;
 	as_node_cdata_to_raw (data);
@@ -967,7 +1139,7 @@ as_node_get_tag (const AsNode *node)
 		return AS_TAG_UNKNOWN;
 
 	/* try to match with a fallback */
-	if (data->tag == AS_TAG_UNKNOWN) {
+	if (!data->is_tag_valid) {
 		tmp = as_tag_data_get_name (data);
 		return as_tag_from_string_full (tmp, AS_TAG_FLAG_USE_FALLBACKS);
 	}
@@ -998,9 +1170,10 @@ as_node_set_data (AsNode *node,
 		return;
 
 	data = (AsNodeData *) node->data;
-	g_free (data->cdata);
-	data->cdata = g_strdup (cdata);
-	data->cdata_escaped = insert_flags & AS_NODE_INSERT_FLAG_PRE_ESCAPED;
+	if (data->is_root_node)
+		return;
+	as_ref_string_assign_safe (&data->cdata, cdata);
+	data->is_cdata_escaped = insert_flags & AS_NODE_INSERT_FLAG_PRE_ESCAPED;
 }
 
 /**
@@ -1018,33 +1191,6 @@ as_node_set_comment (AsNode *node, const gchar *comment)
 	as_node_add_attribute (node, "@comment", comment);
 }
 
-/**
- * as_node_take_data:
- * @node: a #AsNode
- *
- * Gets the node data, e.g. "paragraph text"
- *
- * Return value: string value
- *
- * Since: 0.1.0
- **/
-gchar *
-as_node_take_data (const AsNode *node)
-{
-	AsNodeData *data;
-	gchar *tmp;
-
-	if (node->data == NULL)
-		return NULL;
-	data = (AsNodeData *) node->data;
-	if (data->cdata == NULL || data->cdata[0] == '\0')
-		return NULL;
-	as_node_cdata_to_raw (data);
-	tmp = data->cdata;
-	data->cdata = NULL;
-	return tmp;
-}
-
 /**
  * as_node_get_attribute_as_int:
  * @node: a #AsNode
@@ -1128,38 +1274,6 @@ as_node_get_attribute (const AsNode *node, const gchar *key)
 	return as_node_attr_lookup (data, key);
 }
 
-/**
- * as_node_take_attribute: (skip)
- * @node: a #AsNode
- * @key: the attribute key
- *
- * Gets a node attribute, e.g. "false"
- *
- * Return value: string value
- *
- * Since: 0.1.2
- **/
-gchar *
-as_node_take_attribute (const AsNode *node, const gchar *key)
-{
-	AsNodeAttr *attr;
-	AsNodeData *data;
-	gchar *tmp;
-
-	g_return_val_if_fail (node != NULL, NULL);
-	g_return_val_if_fail (key != NULL, NULL);
-
-	if (node->data == NULL)
-		return NULL;
-	data = (AsNodeData *) node->data;
-	attr = as_node_attr_find (data, key);
-	if (attr == NULL)
-		return NULL;
-	tmp = attr->value;
-	attr->value = NULL;
-	return tmp;
-}
-
 /**
  * as_node_remove_attribute: (skip)
  * @node: a #AsNode
@@ -1204,7 +1318,7 @@ as_node_add_attribute (AsNode *node,
 		       const gchar *value)
 {
 	AsNodeData *data;
-	AsNodeAttr *attr;
+	AsNode *root = g_node_get_root (node);
 
 	g_return_if_fail (node != NULL);
 	g_return_if_fail (key != NULL);
@@ -1212,8 +1326,7 @@ as_node_add_attribute (AsNode *node,
 	if (node->data == NULL)
 		return;
 	data = (AsNodeData *) node->data;
-	attr = as_node_attr_insert (data, key, NULL);
-	attr->value = g_strdup (value);
+	as_node_attr_insert (root, data, key, value);
 }
 
 /**
@@ -1322,9 +1435,9 @@ as_node_find_with_attribute (AsNode *root, const gchar *path,
 static gchar *
 as_node_insert_line_breaks (const gchar *text, guint break_len)
 {
-	GString *str;
 	guint i;
 	gssize new_len;
+	g_autoptr(GString) str = NULL;
 
 	/* allocate long enough for the string, plus the extra newlines */
 	new_len = (gssize) (strlen (text) * (break_len + 1) / break_len);
@@ -1336,7 +1449,7 @@ as_node_insert_line_breaks (const gchar *text, guint break_len)
 	for (i = break_len + 1; i < str->len; i += break_len + 1)
 		g_string_insert (str, (gssize) i, "\n");
 	g_string_append (str, "\n");
-	return g_string_free (str, FALSE);
+	return as_ref_string_new_copy_with_length (str->str, str->len);;
 }
 
 /**
@@ -1363,20 +1476,21 @@ as_node_insert (AsNode *parent,
 	const gchar *key;
 	const gchar *value;
 	AsNodeData *data;
+	AsNode *root = g_node_get_root (parent);
 	guint i;
 	va_list args;
 
 	g_return_val_if_fail (name != NULL, NULL);
 
 	data = g_slice_new0 (AsNodeData);
-	as_node_data_set_name (data, name, insert_flags);
+	as_node_data_set_name (root, data, name, insert_flags);
 	if (cdata != NULL) {
 		if (insert_flags & AS_NODE_INSERT_FLAG_BASE64_ENCODED)
 			data->cdata = as_node_insert_line_breaks (cdata, 76);
 		else
-			data->cdata = g_strdup (cdata);
+			data->cdata = as_ref_string_new (cdata);
 	}
-	data->cdata_escaped = insert_flags & AS_NODE_INSERT_FLAG_PRE_ESCAPED;
+	data->is_cdata_escaped = insert_flags & AS_NODE_INSERT_FLAG_PRE_ESCAPED;
 
 	/* process the attrs valist */
 	va_start (args, insert_flags);
@@ -1387,7 +1501,7 @@ as_node_insert (AsNode *parent,
 		value = va_arg (args, const gchar *);
 		if (value == NULL)
 			break;
-		as_node_attr_insert (data, key, value);
+		as_node_attr_insert (root, data, key, value);
 	}
 	va_end (args);
 
@@ -1418,6 +1532,7 @@ as_node_insert_localized (AsNode *parent,
 			  AsNodeInsertFlags insert_flags)
 {
 	AsNodeData *data;
+	AsNode *root = g_node_get_root (parent);
 	GList *l;
 	const gchar *key;
 	const gchar *value;
@@ -1431,13 +1546,14 @@ as_node_insert_localized (AsNode *parent,
 	if (value_c == NULL)
 		return;
 	data = g_slice_new0 (AsNodeData);
-	as_node_data_set_name (data, name, insert_flags);
+	as_node_data_set_name (root, data, name, insert_flags);
 	if (insert_flags & AS_NODE_INSERT_FLAG_NO_MARKUP) {
-		data->cdata = as_markup_convert_simple (value_c, NULL);
-		data->cdata_escaped = FALSE;
+		g_autofree gchar *tmp = as_markup_convert_simple (value_c, NULL);
+		data->cdata = as_ref_string_new_copy (tmp);
+		data->is_cdata_escaped = FALSE;
 	} else {
-		data->cdata = g_strdup (value_c);
-		data->cdata_escaped = insert_flags & AS_NODE_INSERT_FLAG_PRE_ESCAPED;
+		data->cdata = as_ref_string_new (value_c);
+		data->is_cdata_escaped = insert_flags & AS_NODE_INSERT_FLAG_PRE_ESCAPED;
 	}
 	g_node_insert_data (parent, -1, data);
 
@@ -1455,14 +1571,15 @@ as_node_insert_localized (AsNode *parent,
 		    g_strcmp0 (value_c, value) == 0)
 			continue;
 		data = g_slice_new0 (AsNodeData);
-		as_node_attr_insert (data, "xml:lang", key);
-		as_node_data_set_name (data, name, insert_flags);
+		as_node_attr_insert (root, data, "xml:lang", key);
+		as_node_data_set_name (root, data, name, insert_flags);
 		if (insert_flags & AS_NODE_INSERT_FLAG_NO_MARKUP) {
-			data->cdata = as_markup_convert_simple (value, NULL);
-			data->cdata_escaped = FALSE;
+			g_autofree gchar *tmp = as_markup_convert_simple (value, NULL);
+			data->cdata = as_ref_string_new_copy (tmp);
+			data->is_cdata_escaped = FALSE;
 		} else {
-			data->cdata = g_strdup (value);
-			data->cdata_escaped = insert_flags & AS_NODE_INSERT_FLAG_PRE_ESCAPED;
+			data->cdata = as_ref_string_new (value);
+			data->is_cdata_escaped = insert_flags & AS_NODE_INSERT_FLAG_PRE_ESCAPED;
 		}
 		g_node_insert_data (parent, -1, data);
 	}
@@ -1488,6 +1605,7 @@ as_node_insert_hash (AsNode *parent,
 		     AsNodeInsertFlags insert_flags)
 {
 	AsNodeData *data;
+	AsNode *root = g_node_get_root (parent);
 	GList *l;
 	GList *list;
 	const gchar *key;
@@ -1502,15 +1620,15 @@ as_node_insert_hash (AsNode *parent,
 		key = l->data;
 		value = g_hash_table_lookup (hash, key);
 		data = g_slice_new0 (AsNodeData);
-		as_node_data_set_name (data, name, insert_flags);
-		data->cdata = g_strdup (!swapped ? value : key);
-		data->cdata_escaped = insert_flags & AS_NODE_INSERT_FLAG_PRE_ESCAPED;
+		as_node_data_set_name (root, data, name, insert_flags);
+		data->cdata = as_ref_string_new (!swapped ? value : key);
+		data->is_cdata_escaped = insert_flags & AS_NODE_INSERT_FLAG_PRE_ESCAPED;
 		if (!swapped) {
 			if (key != NULL && key[0] != '\0')
-				as_node_attr_insert (data, attr_key, key);
+				as_node_attr_insert (root, data, attr_key, key);
 		} else {
 			if (value != NULL && value[0] != '\0')
-				as_node_attr_insert (data, attr_key, value);
+				as_node_attr_insert (root, data, attr_key, value);
 		}
 		g_node_insert_data (parent, -1, data);
 	}
@@ -1537,6 +1655,7 @@ as_node_get_localized (const AsNode *node, const gchar *key)
 	const gchar *data_localized;
 	GHashTable *hash = NULL;
 	AsNode *tmp;
+	g_autoptr(AsRefString) xml_lang_c = as_ref_string_new_static ("C");
 
 	/* does it exist? */
 	tmp = as_node_get_child_node (node, key, NULL, NULL);
@@ -1545,7 +1664,8 @@ as_node_get_localized (const AsNode *node, const gchar *key)
 	data_unlocalized = as_node_get_data (tmp);
 
 	/* find a node called name */
-	hash = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
+	hash = g_hash_table_new_full (g_str_hash, g_str_equal,
+				      (GDestroyNotify) as_ref_string_unref, NULL);
 	for (tmp = node->children; tmp != NULL; tmp = tmp->next) {
 		data = tmp->data;
 		if (data == NULL)
@@ -1563,7 +1683,7 @@ as_node_get_localized (const AsNode *node, const gchar *key)
 		if (xml_lang != NULL && g_strcmp0 (data_unlocalized, data_localized) == 0)
 			continue;
 		g_hash_table_insert (hash,
-				     g_strdup (xml_lang != NULL ? xml_lang : "C"),
+				     as_ref_string_ref (xml_lang != NULL ? xml_lang : xml_lang_c),
 				     (gpointer) data_localized);
 	}
 	return hash;
@@ -1812,16 +1932,16 @@ as_node_get_localized_unwrap_type_ul (const AsNode *node,
 gchar *
 as_node_fix_locale (const gchar *locale)
 {
-	gchar *tmp;
+	AsRefString *tmp;
 
 	if (locale == NULL)
-		return g_strdup ("C");
+		return as_ref_string_new_static ("C");
 	if (g_strcmp0 (locale, "xx") == 0)
 		return NULL;
 	if (g_strcmp0 (locale, "x-test") == 0)
 		return NULL;
-	tmp = g_strdup (locale);
-	g_strdelimit (tmp, "-", '_');
+	tmp = as_ref_string_new (locale);
+	g_strdelimit (tmp, "-", '_'); // FIXME: might be evil
 	return tmp;
 }
 
@@ -1903,7 +2023,9 @@ as_node_get_localized_unwrap (const AsNode *node, GError **error)
 	}
 
 	/* copy into a hash table of the correct size */
-	results = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, g_free);
+	results = g_hash_table_new_full (g_str_hash, g_str_equal,
+					 (GDestroyNotify) as_ref_string_unref,
+					 (GDestroyNotify) as_ref_string_unref);
 	keys = g_hash_table_get_keys (hash);
 	for (l = keys; l != NULL; l = l->next) {
 		gchar *locale_fixed;
@@ -1914,7 +2036,7 @@ as_node_get_localized_unwrap (const AsNode *node, GError **error)
 		str = g_hash_table_lookup (hash, xml_lang);
 		g_hash_table_insert (results,
 				     locale_fixed,
-				     g_strdup (str->str));
+				     as_ref_string_new (str->str));
 	}
 	return results;
 }
@@ -1925,7 +2047,7 @@ struct _AsNodeContext {
 	AsAppSourceKind	 output;
 	gdouble		 version;
 	gboolean	 output_trusted;
-	gchar		*media_base_url;
+	AsRefString	*media_base_url;
 };
 
 /**
@@ -1948,6 +2070,24 @@ as_node_context_new (void)
 	return ctx;
 }
 
+/**
+ * as_node_context_free: (skip)
+ * @ctx: a #AsNodeContext.
+ *
+ * Frees the node context.
+ *
+ * Since: 0.6.4
+ **/
+void
+as_node_context_free (AsNodeContext *ctx)
+{
+	if (ctx == NULL)
+		return;
+	if (ctx->media_base_url != NULL)
+		as_ref_string_unref (ctx->media_base_url);
+	g_free (ctx);
+}
+
 /**
  * as_node_context_get_version: (skip)
  * @ctx: a #AsNodeContext.
@@ -2098,6 +2238,5 @@ as_node_context_get_media_base_url (AsNodeContext *ctx)
 void
 as_node_context_set_media_base_url (AsNodeContext *ctx, const gchar *url)
 {
-	g_free (ctx->media_base_url);
-	ctx->media_base_url = g_strdup (url);
+	as_ref_string_assign_safe (&ctx->media_base_url, url);
 }
