@@ -33,12 +33,15 @@ struct _AsProfile
 	GMutex		 mutex;
 	GThread		*unthreaded;
 	guint		 autodump_id;
+	guint		 autoprune_duration;
+	guint		 duration_min;
 };
 
 typedef struct {
 	gchar		*id;
 	gint64		 time_start;
 	gint64		 time_stop;
+	gboolean	 threaded;
 } AsProfileItem;
 
 G_DEFINE_TYPE (AsProfile, as_profile, G_TYPE_OBJECT)
@@ -74,6 +77,139 @@ as_profile_item_find (GPtrArray *array, const gchar *id)
 	return NULL;
 }
 
+/**
+ * as_profile_task_set_threaded:
+ * @ptask: A #AsProfileTask
+ * @threaded: boolean
+ *
+ * Sets if the profile task is threaded so it can be printed differently in
+ * in the profile output.
+ *
+ * Since: 0.6.5
+ **/
+void
+as_profile_task_set_threaded (AsProfileTask *ptask, gboolean threaded)
+{
+	AsProfileItem *item;
+	item = as_profile_item_find (ptask->profile->current, ptask->id);
+	if (item == NULL)
+		return;
+	item->threaded = threaded;
+}
+
+static void
+as_profile_prune_safe (AsProfile *profile, guint duration)
+{
+	gint64 now;
+	guint i;
+	g_autoptr(GPtrArray) removal = g_ptr_array_new ();
+
+	/* find all events older than duration */
+	now = g_get_real_time () / 1000;
+	for (i = 0; i < profile->archived->len; i++) {
+		AsProfileItem *item = g_ptr_array_index (profile->archived, i);
+		if (now - item->time_start / 1000 > duration)
+			g_ptr_array_add (removal, item);
+	}
+
+	/* remove each one */
+	for (i = 0; i < removal->len; i++) {
+		AsProfileItem *item = g_ptr_array_index (removal, i);
+		g_ptr_array_remove (profile->archived, item);
+	}
+}
+
+static gint
+as_profile_sort_cb (gconstpointer a, gconstpointer b)
+{
+	AsProfileItem *item_a = *((AsProfileItem **) a);
+	AsProfileItem *item_b = *((AsProfileItem **) b);
+	if (item_a->time_start < item_b->time_start)
+		return -1;
+	if (item_a->time_start > item_b->time_start)
+		return 1;
+	return 0;
+}
+
+static const gchar *
+as_profile_get_item_printable (AsProfileItem *item)
+{
+	if (item->threaded)
+		return "\033[1m#\033[0m";
+	return "#";
+}
+
+static void
+as_profile_dump_safe (AsProfile *profile)
+{
+	AsProfileItem *item;
+	gint64 time_start = G_MAXINT64;
+	gint64 time_stop = 0;
+	gint64 time_ms;
+	guint console_width = 86;
+	guint i;
+	guint j;
+	gdouble scale;
+	guint bar_offset;
+	guint bar_length;
+
+	/* nothing to show */
+	if (profile->archived->len == 0)
+		return;
+
+	/* get the start and end times */
+	for (i = 0; i < profile->archived->len; i++) {
+		item = g_ptr_array_index (profile->archived, i);
+		if (item->time_start < time_start)
+			time_start = item->time_start;
+		if (item->time_stop > time_stop)
+			time_stop = item->time_stop;
+	}
+	scale = (gdouble) console_width / (gdouble) ((time_stop - time_start) / 1000);
+
+	/* sort the list */
+	g_ptr_array_sort (profile->archived, as_profile_sort_cb);
+
+	/* dump a list of what happened when */
+	for (i = 0; i < profile->archived->len; i++) {
+		const gchar *printable;
+		item = g_ptr_array_index (profile->archived, i);
+		time_ms = (item->time_stop - item->time_start) / 1000;
+		if (time_ms < profile->duration_min)
+			continue;
+
+		/* print a timechart of what we've done */
+		bar_offset = (guint) (scale * (gdouble) (item->time_start -
+							 time_start) / 1000);
+		for (j = 0; j < bar_offset; j++)
+			g_printerr (" ");
+		bar_length = (guint) (scale * (gdouble) time_ms);
+		if (bar_length == 0)
+			bar_length = 1;
+		printable = as_profile_get_item_printable (item);
+		for (j = 0; j < bar_length; j++)
+			g_printerr ("%s", printable);
+		for (j = bar_offset + bar_length; j < console_width + 1; j++)
+			g_printerr (" ");
+		g_printerr ("@%04" G_GINT64_FORMAT "ms ",
+			    (item->time_stop - time_start) / 1000);
+		g_printerr ("%s %" G_GINT64_FORMAT "ms\n", item->id, time_ms);
+	}
+
+	/* not all complete */
+	if (profile->current->len > 0) {
+		for (i = 0; i < profile->current->len; i++) {
+			item = g_ptr_array_index (profile->current, i);
+			item->time_stop = g_get_real_time ();
+			for (j = 0; j < console_width; j++)
+				g_print ("$");
+			time_ms = (item->time_stop - item->time_start) / 1000;
+			g_printerr (" @????ms %s %" G_GINT64_FORMAT "ms\n",
+				    item->id, time_ms);
+		}
+	}
+}
+
 /**
  * as_profile_start:
  * @profile: A #AsProfile
@@ -120,6 +256,10 @@ as_profile_start_literal (AsProfile *profile, const gchar *id)
 	g_return_val_if_fail (AS_IS_PROFILE (profile), NULL);
 	g_return_val_if_fail (id != NULL, NULL);
 
+	/* autoprune old data */
+	if (profile->autoprune_duration != 0)
+		as_profile_prune_safe (profile, profile->autoprune_duration);
+
 	/* only use the thread ID when not using the main thread */
 	self = g_thread_self ();
 	if (self != profile->unthreaded) {
@@ -131,7 +271,7 @@ as_profile_start_literal (AsProfile *profile, const gchar *id)
 	/* already started */
 	item = as_profile_item_find (profile->current, id_thr);
 	if (item != NULL) {
-		as_profile_dump (profile);
+		as_profile_dump_safe (profile);
 		g_warning ("Already a started task for %s", id_thr);
 		return NULL;
 	}
@@ -210,18 +350,6 @@ as_profile_task_free (AsProfileTask *ptask)
 	g_free (ptask);
 }
 
-static gint
-as_profile_sort_cb (gconstpointer a, gconstpointer b)
-{
-	AsProfileItem *item_a = *((AsProfileItem **) a);
-	AsProfileItem *item_b = *((AsProfileItem **) b);
-	if (item_a->time_start < item_b->time_start)
-		return -1;
-	if (item_a->time_start > item_b->time_start)
-		return 1;
-	return 0;
-}
-
 /**
  * as_profile_clear:
  * @profile: A #AsProfile
@@ -233,9 +361,58 @@ as_profile_sort_cb (gconstpointer a, gconstpointer b)
 void
 as_profile_clear (AsProfile *profile)
 {
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&profile->mutex);
 	g_ptr_array_set_size (profile->archived, 0);
 }
 
+/**
+ * as_profile_prune:
+ * @profile: A #AsProfile
+ * @duration: A time in ms
+ *
+ * Clears the list of profiled events older than @duration.
+ *
+ * Since: 0.6.4
+ **/
+void
+as_profile_prune (AsProfile *profile, guint duration)
+{
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&profile->mutex);
+	g_return_if_fail (AS_IS_PROFILE (profile));
+	as_profile_prune_safe (profile, duration);
+}
+
+/**
+ * as_profile_set_autoprune:
+ * @profile: A #AsProfile
+ * @duration: A time in ms
+ *
+ * Automatically prunes events older than @duration when new ones are added.
+ *
+ * Since: 0.6.4
+ **/
+void
+as_profile_set_autoprune (AsProfile *profile, guint duration)
+{
+	profile->autoprune_duration = duration;
+	as_profile_prune (profile, duration);
+}
+
+/**
+ * as_profile_set_duration_min:
+ * @profile: A #AsProfile
+ * @duration_min: A time in ms
+ *
+ * Sets the smallest recordable task duration.
+ *
+ * Since: 0.6.5
+ **/
+void
+as_profile_set_duration_min (AsProfile *profile, guint duration_min)
+{
+	profile->duration_min = duration_min;
+}
+
 /**
  * as_profile_dump:
  * @profile: A #AsProfile
@@ -247,72 +424,9 @@ as_profile_clear (AsProfile *profile)
 void
 as_profile_dump (AsProfile *profile)
 {
-	AsProfileItem *item;
-	gint64 time_start = G_MAXINT64;
-	gint64 time_stop = 0;
-	gint64 time_ms;
-	guint console_width = 86;
-	guint i;
-	guint j;
-	gdouble scale;
-	guint bar_offset;
-	guint bar_length;
-
+	g_autoptr(GMutexLocker) locker = g_mutex_locker_new (&profile->mutex);
 	g_return_if_fail (AS_IS_PROFILE (profile));
-
-	/* nothing to show */
-	if (profile->archived->len == 0)
-		return;
-
-	/* get the start and end times */
-	for (i = 0; i < profile->archived->len; i++) {
-		item = g_ptr_array_index (profile->archived, i);
-		if (item->time_start < time_start)
-			time_start = item->time_start;
-		if (item->time_stop > time_stop)
-			time_stop = item->time_stop;
-	}
-	scale = (gdouble) console_width / (gdouble) ((time_stop - time_start) / 1000);
-
-	/* sort the list */
-	g_ptr_array_sort (profile->archived, as_profile_sort_cb);
-
-	/* dump a list of what happened when */
-	for (i = 0; i < profile->archived->len; i++) {
-		item = g_ptr_array_index (profile->archived, i);
-		time_ms = (item->time_stop - item->time_start) / 1000;
-		if (time_ms < 5)
-			continue;
-
-		/* print a timechart of what we've done */
-		bar_offset = (guint) (scale * (gdouble) (item->time_start -
-							 time_start) / 1000);
-		for (j = 0; j < bar_offset; j++)
-			g_print (" ");
-		bar_length = (guint) (scale * (gdouble) time_ms);
-		if (bar_length == 0)
-			bar_length = 1;
-		for (j = 0; j < bar_length; j++)
-			g_print ("#");
-		for (j = bar_offset + bar_length; j < console_width + 1; j++)
-			g_print (" ");
-		g_print ("@%04" G_GINT64_FORMAT "ms ",
-			 (item->time_stop - time_start) / 1000);
-		g_print ("%s %" G_GINT64_FORMAT "ms\n", item->id, time_ms);
-	}
-
-	/* not all complete */
-	if (profile->current->len > 0) {
-		for (i = 0; i < profile->current->len; i++) {
-			item = g_ptr_array_index (profile->current, i);
-			item->time_stop = g_get_real_time ();
-			for (j = 0; j < console_width; j++)
-				g_print ("$");
-			time_ms = (item->time_stop - item->time_start) / 1000;
-			g_print (" @????ms %s %" G_GINT64_FORMAT "ms\n",
-				 item->id, time_ms);
-		}
-	}
+	as_profile_dump_safe (profile);
 }
 
 static gboolean
@@ -365,6 +479,7 @@ as_profile_class_init (AsProfileClass *klass)
 static void
 as_profile_init (AsProfile *profile)
 {
+	profile->duration_min = 5;
 	profile->current = g_ptr_array_new ();
 	profile->unthreaded = g_thread_self ();
 	profile->archived = g_ptr_array_new_with_free_func ((GDestroyNotify) as_profile_item_free);
