@@ -41,6 +41,8 @@
 #include "as-problem.h"
 #include "as-profile.h"
 #include "as-monitor.h"
+#include "as-ref-string.h"
+#include "as-stemmer.h"
 #include "as-store.h"
 #include "as-utils-private.h"
 #include "as-yaml.h"
@@ -71,13 +73,16 @@ typedef struct
 	AsMonitor		*monitor;
 	GHashTable		*metadata_indexes;	/* GHashTable{key} */
 	GHashTable		*appinfo_dirs;	/* GHashTable{path:AsStorePathData} */
+	GHashTable		*search_blacklist;	/* GHashTable{AsRefString:1} */
 	AsStoreAddFlags		 add_flags;
 	AsStoreWatchFlags	 watch_flags;
 	AsStoreProblems		 problems;
+	AsAppSearchMatch	 search_match;
 	guint32			 filter;
 	guint			 changed_block_refcnt;
 	gboolean		 is_pending_changed_signal;
 	AsProfile		*profile;
+	AsStemmer		*stemmer;
 } AsStorePrivate;
 
 typedef struct {
@@ -89,6 +94,9 @@ G_DEFINE_TYPE_WITH_PRIVATE (AsStore, as_store, G_TYPE_OBJECT)
 
 enum {
 	SIGNAL_CHANGED,
+	SIGNAL_APP_ADDED,
+	SIGNAL_APP_REMOVED,
+	SIGNAL_APP_CHANGED,
 	SIGNAL_LAST
 };
 
@@ -109,6 +117,7 @@ static gboolean	as_store_from_file_internal (AsStore *store,
 					     GFile *file,
 					     AsAppScope scope,
 					     const gchar *arch,
+					     AsStoreLoadFlags load_flags,
 					     AsStoreWatchFlags watch_flags,
 					     GCancellable *cancellable,
 					     GError **error);
@@ -125,12 +134,14 @@ as_store_finalize (GObject *object)
 	g_ptr_array_unref (priv->array);
 	g_object_unref (priv->monitor);
 	g_object_unref (priv->profile);
+	g_object_unref (priv->stemmer);
 	g_hash_table_unref (priv->hash_id);
 	g_hash_table_unref (priv->hash_merge_id);
 	g_hash_table_unref (priv->hash_unique_id);
 	g_hash_table_unref (priv->hash_pkgname);
 	g_hash_table_unref (priv->metadata_indexes);
 	g_hash_table_unref (priv->appinfo_dirs);
+	g_hash_table_unref (priv->search_blacklist);
 
 	G_OBJECT_CLASS (as_store_parent_class)->finalize (object);
 }
@@ -142,7 +153,7 @@ as_store_class_init (AsStoreClass *klass)
 
 	/**
 	 * AsStore::changed:
-	 * @device: the #AsStore instance that emitted the signal
+	 * @store: the #AsStore instance that emitted the signal
 	 *
 	 * The ::changed signal is emitted when components have been added
 	 * or removed from the store.
@@ -156,6 +167,57 @@ as_store_class_init (AsStoreClass *klass)
 			      NULL, NULL, g_cclosure_marshal_VOID__VOID,
 			      G_TYPE_NONE, 0);
 
+	/**
+	 * AsStore::app-added:
+	 * @store: the #AsStore instance that emitted the signal
+	 * @app: the #AsApp instance
+	 *
+	 * The ::app-added signal is emitted when a component has been added to
+	 * the store.
+	 *
+	 * Since: 0.6.5
+	 **/
+	signals [SIGNAL_APP_ADDED] =
+		g_signal_new ("app-added",
+			      G_TYPE_FROM_CLASS (object_class), G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (AsStoreClass, app_added),
+			      NULL, NULL, g_cclosure_marshal_VOID__OBJECT,
+			      G_TYPE_NONE, 1, AS_TYPE_APP);
+
+	/**
+	 * AsStore::app-removed:
+	 * @store: the #AsStore instance that emitted the signal
+	 * @app: the #AsApp instance
+	 *
+	 * The ::app-removed signal is emitted when a component has been removed
+	 * from the store.
+	 *
+	 * Since: 0.6.5
+	 **/
+	signals [SIGNAL_APP_REMOVED] =
+		g_signal_new ("app-removed",
+			      G_TYPE_FROM_CLASS (object_class), G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (AsStoreClass, app_removed),
+			      NULL, NULL, g_cclosure_marshal_VOID__OBJECT,
+			      G_TYPE_NONE, 1, AS_TYPE_APP);
+
+	/**
+	 * AsStore::app-changed:
+	 * @store: the #AsStore instance that emitted the signal
+	 * @app: the #AsApp instance
+	 *
+	 * The ::app-changed signal is emitted when a component has been changed
+	 * in the store.
+	 *
+	 * Since: 0.6.5
+	 **/
+	signals [SIGNAL_APP_CHANGED] =
+		g_signal_new ("app-changed",
+			      G_TYPE_FROM_CLASS (object_class), G_SIGNAL_RUN_LAST,
+			      G_STRUCT_OFFSET (AsStoreClass, app_changed),
+			      NULL, NULL, g_cclosure_marshal_VOID__OBJECT,
+			      G_TYPE_NONE, 1, AS_TYPE_APP);
+
 	object_class->finalize = as_store_finalize;
 }
 
@@ -646,6 +708,7 @@ as_store_get_app_by_id_with_fallbacks (AsStore *store, const gchar *id)
 		{ "file-roller.desktop",	"org.gnome.FileRoller.desktop" },
 		{ "font-manager.desktop",	"org.gnome.FontManager.desktop" },
 		{ "gcalctool.desktop",		"gnome-calculator.desktop" },
+		{ "geary.desktop",		"org.gnome.Geary.desktop" },
 		{ "gedit.desktop",		"org.gnome.gedit.desktop" },
 		{ "glchess.desktop",		"gnome-chess.desktop" },
 		{ "glines.desktop",		"five-or-more.desktop" },
@@ -833,6 +896,9 @@ as_store_remove_app (AsStore *store, AsApp *app)
 	AsStorePrivate *priv = GET_PRIVATE (store);
 	GPtrArray *apps;
 
+	/* emit before removal */
+	g_signal_emit (store, signals[SIGNAL_APP_REMOVED], 0, app);
+
 	/* only remove this specific unique app */
 	apps = g_hash_table_lookup (priv->hash_id, as_app_get_id (app));
 	if (apps != NULL)
@@ -868,6 +934,10 @@ as_store_remove_app_by_id (AsStore *store, const gchar *id)
 		app = g_ptr_array_index (priv->array, i);
 		if (g_strcmp0 (id, as_app_get_id (app)) != 0)
 			continue;
+
+		/* emit before removal */
+		g_signal_emit (store, signals[SIGNAL_APP_REMOVED], 0, app);
+
 		g_ptr_array_remove (priv->array, app);
 		g_hash_table_remove (priv->hash_unique_id,
 				     as_app_get_unique_id (app));
@@ -892,6 +962,37 @@ _as_app_is_perhaps_merge_component (AsApp *app)
 	return TRUE;
 }
 
+/**
+ * as_store_add_apps:
+ * @store: a #AsStore instance.
+ * @apps: (element-type AsApp): an array of apps
+ *
+ * Adds several applications to the store.
+ *
+ * Additionally only applications where the kind is known will be added.
+ *
+ * Since: 0.6.4
+ **/
+void
+as_store_add_apps (AsStore *store, GPtrArray *apps)
+{
+	guint i;
+	_cleanup_uninhibit_ guint32 *tok = NULL;
+
+	g_return_if_fail (AS_IS_STORE (store));
+
+	/* emit once when finished */
+	tok = as_store_changed_inhibit (store);
+	for (i = 0; i < apps->len; i++) {
+		AsApp *app = g_ptr_array_index (apps, i);
+		as_store_add_app (store, app);
+	}
+
+	/* this store has changed */
+	as_store_changed_uninhibit (&tok);
+	as_store_perhaps_emit_changed (store, "add-apps");
+}
+
 /**
  * as_store_add_app:
  * @store: a #AsStore instance.
@@ -962,6 +1063,10 @@ as_store_add_app (AsStore *store, AsApp *app)
 				 as_app_merge_kind_to_string (merge_kind),
 				 id, as_app_get_unique_id (app_tmp));
 			as_app_subsume_full (app_tmp, app, flags);
+
+			/* emit after changes have been made */
+			g_signal_emit (store, signals[SIGNAL_APP_CHANGED],
+				       0, app_tmp);
 		}
 		return;
 	}
@@ -1042,6 +1147,8 @@ as_store_add_app (AsStore *store, AsApp *app)
 				g_debug ("ignoring AppData entry as AppStream exists: %s:%s",
 					 as_app_get_unique_id (app),
 					 as_app_get_unique_id (item));
+				as_app_subsume_full (item, app,
+						     AS_APP_SUBSUME_FLAG_RELEASES);
 				return;
 			}
 			if (as_app_get_source_kind (app) == AS_APP_SOURCE_KIND_DESKTOP &&
@@ -1090,6 +1197,8 @@ as_store_add_app (AsStore *store, AsApp *app)
 		g_debug ("removing %s entry: %s",
 			 as_app_source_kind_to_string (as_app_get_source_kind (item)),
 			 as_app_get_unique_id (item));
+		if (as_app_get_state (item) == AS_APP_STATE_INSTALLED)
+			as_app_set_state (app, AS_APP_STATE_INSTALLED);
 		as_store_remove_app (store, item);
 	}
 
@@ -1116,7 +1225,13 @@ as_store_add_app (AsStore *store, AsApp *app)
 				     g_object_ref (app));
 	}
 
+	/* add helper objects */
+	as_app_set_stemmer (app, priv->stemmer);
+	as_app_set_search_blacklist (app, priv->search_blacklist);
+	as_app_set_search_match (app, priv->search_match);
+
 	/* added */
+	g_signal_emit (store, signals[SIGNAL_APP_ADDED], 0, app);
 	as_store_perhaps_emit_changed (store, "add-app");
 }
 
@@ -1159,6 +1274,7 @@ as_store_match_addons (AsStore *store)
 
 	/* profile */
 	ptask = as_profile_start_literal (priv->profile, "AsStore:match-addons");
+	g_assert (ptask != NULL);
 	for (i = 0; i < priv->array->len; i++) {
 		AsApp *app = g_ptr_array_index (priv->array, i);
 		if (as_app_get_kind (app) != AS_APP_KIND_ADDON)
@@ -1200,6 +1316,7 @@ as_store_from_root (AsStore *store,
 		    const gchar *icon_prefix,
 		    const gchar *source_filename,
 		    const gchar *arch,
+		    AsStoreLoadFlags load_flags,
 		    GError **error)
 {
 	AsStorePrivate *priv = GET_PRIVATE (store);
@@ -1208,13 +1325,16 @@ as_store_from_root (AsStore *store,
 	const gchar *tmp;
 	const gchar *origin_delim = ":";
 	gchar *str;
-	g_autofree AsNodeContext *ctx = NULL;
+	g_autoptr(AsNodeContext) ctx = NULL;
 	g_autofree gchar *icon_path = NULL;
 	g_autofree gchar *id_prefix_app = NULL;
 	g_autofree gchar *origin_app = NULL;
 	g_autofree gchar *origin_app_icons = NULL;
 	_cleanup_uninhibit_ guint32 *tok = NULL;
 	g_autoptr(AsProfileTask) ptask = NULL;
+	g_autoptr(AsRefString) icon_path_str = NULL;
+	g_autoptr(AsRefString) origin_str = NULL;
+	g_autoptr(AsRefString) source_filename_str = NULL;
 
 	g_return_val_if_fail (AS_IS_STORE (store), FALSE);
 
@@ -1225,6 +1345,7 @@ as_store_from_root (AsStore *store,
 
 	/* profile */
 	ptask = as_profile_start_literal (priv->profile, "AsStore:store-from-root");
+	g_assert (ptask != NULL);
 
 	/* emit once when finished */
 	tok = as_store_changed_inhibit (store);
@@ -1278,7 +1399,6 @@ as_store_from_root (AsStore *store,
 		str = g_strrstr (source_basename, ".xml");
 		if (str != NULL) {
 			str[0] = '\0';
-			origin_app = g_strdup (source_basename + 8);
 			origin_app_icons = g_strdup (source_basename);
 		}
 
@@ -1286,6 +1406,7 @@ as_store_from_root (AsStore *store,
 		str = g_strstr_len (source_basename, -1, origin_delim);
 		if (str != NULL) {
 			str[0] = '\0';
+			origin_app = g_strdup (str + 1);
 			id_prefix_app = g_strdup (source_basename);
 		}
 
@@ -1332,6 +1453,14 @@ as_store_from_root (AsStore *store,
 	if (tmp != NULL)
 		as_store_set_builder_id (store, tmp);
 
+	/* create refcounted versions */
+	if (source_filename != NULL)
+		source_filename_str = as_ref_string_new (source_filename);
+	if (origin_app != NULL)
+		origin_str = as_ref_string_new (origin_app);
+	if (icon_path != NULL)
+		icon_path_str = as_ref_string_new (icon_path);
+
 	ctx = as_node_context_new ();
 	for (n = apps->children; n != NULL; n = n->next) {
 		g_autoptr(GError) error_local = NULL;
@@ -1351,8 +1480,8 @@ as_store_from_root (AsStore *store,
 		}
 
 		app = as_app_new ();
-		if (icon_path != NULL)
-			as_app_set_icon_path (app, icon_path);
+		if (icon_path_str != NULL)
+			as_app_set_icon_path (app, icon_path_str);
 		if (arch != NULL)
 			as_app_add_arch (app, arch);
 		as_app_set_scope (app, scope);
@@ -1366,14 +1495,22 @@ as_store_from_root (AsStore *store,
 			return FALSE;
 		}
 
+		/* filter out non-merge types */
+		if (load_flags & AS_STORE_LOAD_FLAG_ONLY_MERGE_APPS) {
+			if (as_app_get_merge_kind (app) != AS_APP_MERGE_KIND_APPEND &&
+			    as_app_get_merge_kind (app) != AS_APP_MERGE_KIND_APPEND) {
+				continue;
+			}
+		}
+
 		/* set the ID prefix */
 		if ((priv->add_flags & AS_STORE_ADD_FLAG_USE_UNIQUE_ID) == 0)
 			as_store_fixup_id_prefix (app, id_prefix_app);
 
-		if (origin_app != NULL)
-			as_app_set_origin (app, origin_app);
-		if (source_filename != NULL)
-			as_app_set_source_file (app, source_filename);
+		if (origin_str != NULL)
+			as_app_set_origin (app, origin_str);
+		if (source_filename_str != NULL)
+			as_app_set_source_file (app, source_filename_str);
 		as_store_add_app (store, app);
 	}
 
@@ -1390,20 +1527,24 @@ as_store_from_root (AsStore *store,
 static gboolean
 as_store_load_yaml_file (AsStore *store,
 			 GFile *file,
+			 AsAppScope scope,
 			 GCancellable *cancellable,
 			 GError **error)
 {
 	AsStorePrivate *priv = GET_PRIVATE (store);
 	AsNode *app_n;
 	AsNode *n;
+	AsYamlFromFlags flags = AS_YAML_FROM_FLAG_NONE;
 	const gchar *tmp;
-	g_autofree AsNodeContext *ctx = NULL;
+	g_autoptr(AsNodeContext) ctx = NULL;
 	g_autofree gchar *icon_path = NULL;
 	g_autoptr(AsYaml) root = NULL;
 	_cleanup_uninhibit_ guint32 *tok = NULL;
 
 	/* load file */
-	root = as_yaml_from_file (file, cancellable, error);
+	if (priv->add_flags & AS_STORE_ADD_FLAG_ONLY_NATIVE_LANGS)
+		flags |= AS_YAML_FROM_FLAG_ONLY_NATIVE_LANGS;
+	root = as_yaml_from_file (file, flags, cancellable, error);
 	if (root == NULL)
 		return FALSE;
 
@@ -1458,6 +1599,7 @@ as_store_load_yaml_file (AsStore *store,
 
 		if (icon_path != NULL)
 			as_app_set_icon_path (app, icon_path);
+		as_app_set_scope (app, scope);
 		as_app_set_source_kind (app, AS_APP_SOURCE_KIND_APPSTREAM);
 		if (!as_app_node_parse_dep11 (app, app_n, ctx, error))
 			return FALSE;
@@ -1536,6 +1678,7 @@ as_store_watch_source_added (AsStore *store, const gchar *filename)
 					  file,
 					  path_data->scope,
 					  path_data->arch,
+					  AS_STORE_LOAD_FLAG_NONE,
 					  AS_STORE_WATCH_FLAG_NONE,
 					  NULL, /* cancellable */
 					  &error)){
@@ -1650,11 +1793,13 @@ as_store_from_file_internal (AsStore *store,
 			     GFile *file,
 			     AsAppScope scope,
 			     const gchar *arch,
+			     AsStoreLoadFlags load_flags,
 			     AsStoreWatchFlags watch_flags,
 			     GCancellable *cancellable,
 			     GError **error)
 {
 	AsStorePrivate *priv = GET_PRIVATE (store);
+	AsNodeFromXmlFlags flags = AS_NODE_FROM_XML_FLAG_LITERAL_TEXT;
 	g_autofree gchar *filename = NULL;
 	g_autofree gchar *icon_prefix = NULL;
 	g_autoptr(GError) error_local = NULL;
@@ -1664,13 +1809,20 @@ as_store_from_file_internal (AsStore *store,
 	g_return_val_if_fail (AS_IS_STORE (store), FALSE);
 
 	/* profile */
-	ptask = as_profile_start_literal (priv->profile, "AsStore:store-from-file");
+	filename = g_file_get_path (file);
+	ptask = as_profile_start (priv->profile,
+				  "AsStore:store-from-file{%s}",
+				  filename);
+	g_assert (ptask != NULL);
 
 	/* a DEP-11 file */
-	filename = g_file_get_path (file);
-	if (g_strstr_len (filename, -1, ".yml") != NULL)
-		return as_store_load_yaml_file (store, file, cancellable, error);
-
+	if (g_strstr_len (filename, -1, ".yml") != NULL) {
+		return as_store_load_yaml_file (store,
+						file,
+						scope,
+						cancellable,
+						error);
+	}
 #ifdef HAVE_GCAB
 	/* a cab archive */
 	if (g_str_has_suffix (filename, ".cab"))
@@ -1678,10 +1830,9 @@ as_store_from_file_internal (AsStore *store,
 #endif
 
 	/* an AppStream XML file */
-	root = as_node_from_file (file,
-				  AS_NODE_FROM_XML_FLAG_LITERAL_TEXT,
-				  cancellable,
-				  &error_local);
+	if (priv->add_flags & AS_STORE_ADD_FLAG_ONLY_NATIVE_LANGS)
+		flags |= AS_NODE_FROM_XML_FLAG_ONLY_NATIVE_LANGS;
+	root = as_node_from_file (file, flags, cancellable, &error_local);
 	if (root == NULL) {
 		g_set_error (error,
 			     AS_STORE_ERROR,
@@ -1704,7 +1855,8 @@ as_store_from_file_internal (AsStore *store,
 	/* icon prefix is the directory the XML has been found in */
 	icon_prefix = g_path_get_dirname (filename);
 	return as_store_from_root (store, root, scope,
-				   icon_prefix, filename, arch, error);
+				   icon_prefix, filename, arch, load_flags,
+				   error);
 }
 
 /**
@@ -1738,6 +1890,7 @@ as_store_from_file (AsStore *store,
 	return as_store_from_file_internal (store, file,
 					    AS_APP_SCOPE_UNKNOWN,
 					    NULL, /* arch */
+					    AS_STORE_LOAD_FLAG_NONE,
 					    priv->watch_flags,
 					    cancellable, error);
 }
@@ -1821,6 +1974,8 @@ as_store_from_xml (AsStore *store,
 		   const gchar *icon_root,
 		   GError **error)
 {
+	AsStorePrivate *priv = GET_PRIVATE (store);
+	AsNodeFromXmlFlags flags = AS_NODE_FROM_XML_FLAG_LITERAL_TEXT;
 	g_autoptr(GError) error_local = NULL;
 	g_autoptr(AsNode) root = NULL;
 
@@ -1831,9 +1986,10 @@ as_store_from_xml (AsStore *store,
 	if (data[0] == '\0')
 		return TRUE;
 
-	root = as_node_from_xml (data,
-				 AS_NODE_FROM_XML_FLAG_LITERAL_TEXT,
-				 &error_local);
+	/* load XML data */
+	if (priv->add_flags & AS_STORE_ADD_FLAG_ONLY_NATIVE_LANGS)
+		flags |= AS_NODE_FROM_XML_FLAG_ONLY_NATIVE_LANGS;
+	root = as_node_from_xml (data, flags, &error_local);
 	if (root == NULL) {
 		g_set_error (error,
 			     AS_STORE_ERROR,
@@ -1847,6 +2003,7 @@ as_store_from_xml (AsStore *store,
 				   icon_root,
 				   NULL, /* filename */
 				   NULL, /* arch */
+				   AS_STORE_LOAD_FLAG_NONE,
 				   error);
 }
 
@@ -1947,7 +2104,7 @@ as_store_to_xml (AsStore *store, AsNodeToXmlFlags flags)
 	gboolean output_trusted = FALSE;
 	guint i;
 	gchar version[6];
-	g_autofree AsNodeContext *ctx = NULL;
+	g_autoptr(AsNodeContext) ctx = NULL;
 
 	/* check categories of apps about to be written */
 	as_store_check_apps_for_veto (store);
@@ -2354,27 +2511,30 @@ as_store_load_app_info_file (AsStore *store,
 			     AsAppScope scope,
 			     const gchar *path_xml,
 			     const gchar *arch,
+			     AsStoreLoadFlags flags,
 			     GCancellable *cancellable,
 			     GError **error)
 {
-	AsStorePrivate *priv = GET_PRIVATE (store);
 	g_autoptr(GFile) file = NULL;
-	g_autoptr(AsProfileTask) ptask = NULL;
 
-	/* profile */
-	ptask = as_profile_start (priv->profile, "AsStore:app-info{%s}", path_xml);
+	/* ignore large compressed files */
+	if (flags & AS_STORE_LOAD_FLAG_ONLY_UNCOMPRESSED &&
+	    g_str_has_suffix (path_xml, ".gz")) {
+		g_debug ("ignoring compressed file %s", path_xml);
+		return TRUE;
+	}
 
 	/* guess this based on the name */
 	if (!as_store_guess_origin_fallback (store, path_xml, error))
 		return FALSE;
 
+	/* load without adding monitor */
 	file = g_file_new_for_path (path_xml);
-	/* Do not monitor the file: assume we are already monitoring its
-	 * directory */
 	return as_store_from_file_internal (store,
 					    file,
 					    scope,
 					    arch,
+					    flags,
 					    AS_STORE_WATCH_FLAG_NONE,
 					    cancellable,
 					    error);
@@ -2390,9 +2550,6 @@ as_store_load_app_info (AsStore *store,
 			GError **error)
 {
 	AsStorePrivate *priv = GET_PRIVATE (store);
-	const gchar *tmp;
-	g_autoptr(GDir) dir = NULL;
-	g_autoptr(GError) error_local = NULL;
 	_cleanup_uninhibit_ guint32 *tok = NULL;
 
 	/* Don't add the same dir twice, we're monitoring it for changes anyway */
@@ -2402,48 +2559,52 @@ as_store_load_app_info (AsStore *store,
 	/* emit once when finished */
 	tok = as_store_changed_inhibit (store);
 
-	/* search all files */
-	if (!g_file_test (path, G_FILE_TEST_EXISTS))
-		return TRUE;
-	dir = g_dir_open (path, 0, &error_local);
-	if (dir == NULL) {
-		if (flags & AS_STORE_LOAD_FLAG_IGNORE_INVALID) {
-			g_warning ("ignoring invalid AppStream path %s: %s",
-				   path, error_local->message);
-			return TRUE;
-		}
-		g_set_error (error,
-			     AS_STORE_ERROR,
-			     AS_STORE_ERROR_FAILED,
-			     "Failed to open %s: %s",
-			     path, error_local->message);
-		return FALSE;
-	}
-
-	while ((tmp = g_dir_read_name (dir)) != NULL) {
-		GError *error_store = NULL;
-		g_autofree gchar *filename_md = NULL;
-		if (g_strcmp0 (tmp, "icons") == 0)
-			continue;
-		filename_md = g_build_filename (path, tmp, NULL);
-		if (!as_store_load_app_info_file (store,
-						  scope,
-						  filename_md,
-						  arch,
-						  cancellable,
-						  &error_store)) {
+	/* search all files, if the location already exists */
+	if (g_file_test (path, G_FILE_TEST_IS_DIR)) {
+		const gchar *tmp;
+		g_autoptr(GDir) dir = NULL;
+		g_autoptr(GError) error_local = NULL;
+		dir = g_dir_open (path, 0, &error_local);
+		if (dir == NULL) {
 			if (flags & AS_STORE_LOAD_FLAG_IGNORE_INVALID) {
-				g_warning ("Ignoring invalid AppStream file %s: %s",
-					   filename_md, error_store->message);
-				g_clear_error (&error_store);
-			} else {
-				g_propagate_error (error, error_store);
-				return FALSE;
+				g_warning ("ignoring invalid AppStream path %s: %s",
+					   path, error_local->message);
+				return TRUE;
+			}
+			g_set_error (error,
+				     AS_STORE_ERROR,
+				     AS_STORE_ERROR_FAILED,
+				     "Failed to open %s: %s",
+				     path, error_local->message);
+			return FALSE;
+		}
+
+		while ((tmp = g_dir_read_name (dir)) != NULL) {
+			GError *error_store = NULL;
+			g_autofree gchar *filename_md = NULL;
+			if (g_strcmp0 (tmp, "icons") == 0)
+				continue;
+			filename_md = g_build_filename (path, tmp, NULL);
+			if (!as_store_load_app_info_file (store,
+							  scope,
+							  filename_md,
+							  arch,
+							  flags,
+							  cancellable,
+							  &error_store)) {
+				if (flags & AS_STORE_LOAD_FLAG_IGNORE_INVALID) {
+					g_warning ("Ignoring invalid AppStream file %s: %s",
+						   filename_md, error_store->message);
+					g_clear_error (&error_store);
+				} else {
+					g_propagate_error (error, error_store);
+					return FALSE;
+				}
 			}
 		}
 	}
 
-	/* watch the directories for changes */
+	/* watch the directories for changes, even if it does not exist yet */
 	as_store_add_path_data (store, path, scope, arch);
 	if (!as_monitor_add_directory (priv->monitor,
 				       path,
@@ -2507,6 +2668,7 @@ as_store_load_installed (AsStore *store,
 
 	/* profile */
 	ptask = as_profile_start (priv->profile, "AsStore:load-installed{%s}", path);
+	g_assert (ptask != NULL);
 
 	dir = g_dir_open (path, 0, error);
 	if (dir == NULL)
@@ -2530,6 +2692,10 @@ as_store_load_installed (AsStore *store,
 	if (flags & AS_STORE_LOAD_FLAG_ALLOW_VETO)
 		parse_flags |= AS_APP_PARSE_FLAG_ALLOW_VETO;
 
+	/* propagate flag */
+	if (priv->add_flags & AS_STORE_ADD_FLAG_ONLY_NATIVE_LANGS)
+		parse_flags |= AS_APP_PARSE_FLAG_ONLY_NATIVE_LANGS;
+
 	while ((tmp = g_dir_read_name (dir)) != NULL) {
 		AsApp *app_tmp;
 		GPtrArray *icons;
@@ -2572,13 +2738,6 @@ as_store_load_installed (AsStore *store,
 				as_icon_set_kind (icon, AS_ICON_KIND_STOCK);
 		}
 
-		/* don't match against non-package apps in the store */
-		if (scope == AS_APP_SCOPE_SYSTEM &&
-		    g_str_has_prefix (path, "/usr/share")) {
-			as_app_set_trust_flags (app, AS_APP_TRUST_FLAG_COMPLETE);
-			as_app_add_pkgname (app, "");
-		}
-
 		/* set the ID prefix */
 		if ((priv->add_flags & AS_STORE_ADD_FLAG_USE_UNIQUE_ID) == 0)
 			as_store_fixup_id_prefix (app, as_app_scope_to_string (scope));
@@ -2662,8 +2821,6 @@ as_store_search_app_info (AsStore *store,
 					 path,
 					 supported_kinds[i],
 					 NULL);
-		if (!g_file_test (dest, G_FILE_TEST_EXISTS))
-			continue;
 		if (!as_store_load_app_info (store, scope, dest, NULL,
 					     flags, cancellable, error))
 			return FALSE;
@@ -2684,6 +2841,7 @@ as_store_search_per_system (AsStore *store,
 
 	/* profile */
 	ptask = as_profile_start_literal (priv->profile, "AsStore:load{per-system}");
+	g_assert (ptask != NULL);
 
 	/* datadir AppStream, AppData and desktop */
 	data_dirs = g_get_system_data_dirs ();
@@ -2767,6 +2925,7 @@ as_store_search_per_user (AsStore *store,
 
 	/* profile */
 	ptask = as_profile_start_literal (priv->profile, "AsStore:load{per-user}");
+	g_assert (ptask != NULL);
 
 	/* AppStream */
 	if ((flags & AS_STORE_LOAD_FLAG_APP_INFO_USER) > 0) {
@@ -2806,6 +2965,50 @@ as_store_search_per_user (AsStore *store,
 	return TRUE;
 }
 
+static void
+as_store_load_search_cache_cb (gpointer data, gpointer user_data)
+{
+	AsApp *app = AS_APP (data);
+	as_app_search_matches (app, NULL);
+}
+
+/**
+ * as_store_load_search_cache:
+ * @store: a #AsStore instance.
+ *
+ * Populates the token cache for all applications in the store. This allows
+ * all the search keywords for all applications in the store to be
+ * pre-processed at one time in multiple threads rather than on demand.
+ *
+ * Note: Calling as_app_search_matches() automatically generates the search
+ * cache for the #AsApp object if it has not already been generated.
+ *
+ * Since: 0.6.5
+ **/
+void
+as_store_load_search_cache (AsStore *store)
+{
+	AsStorePrivate *priv = GET_PRIVATE (store);
+	guint i;
+	GThreadPool *pool;
+	g_autoptr(AsProfileTask) ptask = NULL;
+
+	/* profile */
+	ptask = as_profile_start_literal (priv->profile,
+					  "AsStore:load-token-cache");
+	as_profile_task_set_threaded (ptask, TRUE);
+
+	/* load the token cache for each app in multiple threads */
+	pool = g_thread_pool_new (as_store_load_search_cache_cb,
+				  store, 4, TRUE, NULL);
+	g_assert (pool != NULL);
+	for (i = 0; i < priv->array->len; i++) {
+		AsApp *app = g_ptr_array_index (priv->array, i);
+		g_thread_pool_push (pool, app, NULL);
+	}
+	g_thread_pool_free (pool, FALSE, TRUE);
+}
+
 /**
  * as_store_load:
  * @store: a #AsStore instance.
@@ -2831,6 +3034,7 @@ as_store_load (AsStore *store,
 
 	/* profile */
 	ptask = as_profile_start_literal (priv->profile, "AsStore:load");
+	g_assert (ptask != NULL);
 	tok = as_store_changed_inhibit (store);
 
 	/* per-user locations */
@@ -3142,14 +3346,64 @@ as_store_path_data_free (AsStorePathData *path_data)
 	g_slice_free (AsStorePathData, path_data);
 }
 
+static void
+as_store_create_search_blacklist (AsStore *store)
+{
+	AsStorePrivate *priv = GET_PRIVATE (store);
+	guint i;
+	const gchar *blacklist[] = {
+		"and", "the", "application", "for", "you", "your",
+		"with", "can", "are", "from", "that", "use", "allows", "also",
+		"this", "other", "all", "using", "has", "some", "like", "them",
+		"well", "not", "using", "not", "but", "set", "its", "into",
+		"such", "was", "they", "where", "want", "only", "about",
+		"uses", "font", "features", "designed", "provides", "which",
+		"many", "used", "org", "fonts", "open", "more", "based",
+		"different", "including", "will", "multiple", "out", "have",
+		"each", "when", "need", "most", "both", "their", "even",
+		"way", "several", "been", "while", "very", "add", "under",
+		"what", "those", "much", "either", "currently", "one",
+		"support", "make", "over", "these", "there", "without", "etc",
+		"main",
+		NULL };
+	for (i = 0; blacklist[i] != NULL; i++)  {
+		g_hash_table_insert (priv->search_blacklist,
+				     as_stemmer_process (priv->stemmer, blacklist[i]),
+				     GUINT_TO_POINTER (1));
+	}
+}
+
+/**
+ * as_store_set_search_match:
+ * @store: a #AsStore instance.
+ * @search_match: the API version
+ *
+ * Sets the token match fields. The bitfield given here is used to choose what
+ * is included in the token cache.
+ *
+ * Since: 0.6.5
+ **/
+void
+as_store_set_search_match (AsStore *store, AsAppSearchMatch search_match)
+{
+	AsStorePrivate *priv = GET_PRIVATE (store);
+	priv->search_match = search_match;
+}
+
 static void
 as_store_init (AsStore *store)
 {
 	AsStorePrivate *priv = GET_PRIVATE (store);
 	priv->profile = as_profile_new ();
+	priv->stemmer = as_stemmer_new ();
 	priv->api_version = AS_API_VERSION_NEWEST;
 	priv->array = g_ptr_array_new_with_free_func ((GDestroyNotify) g_object_unref);
 	priv->watch_flags = AS_STORE_WATCH_FLAG_NONE;
+	priv->search_match = AS_APP_SEARCH_MATCH_LAST;
+	priv->search_blacklist = g_hash_table_new_full (g_str_hash,
+							g_str_equal,
+							(GDestroyNotify) as_ref_string_unref,
+							NULL);
 	priv->hash_id = g_hash_table_new_full (g_str_hash,
 					       g_str_equal,
 					       NULL,
@@ -3184,6 +3438,9 @@ as_store_init (AsStore *store)
 							  g_str_equal,
 							  g_free,
 							  (GDestroyNotify) g_hash_table_unref);
+
+	/* add stemmed keywords to the search blacklist */
+	as_store_create_search_blacklist (store);
 }
 
 /**
