@@ -892,6 +892,9 @@ as_utils_find_icon_filename_full (const gchar *destdir,
 				 NULL };
 	g_autofree gchar *prefix = NULL;
 
+	g_return_val_if_fail (destdir != NULL, NULL);
+	g_return_val_if_fail (search != NULL, NULL);
+
 	/* fallback */
 	if (destdir == NULL)
 		destdir = "";
@@ -1270,35 +1273,15 @@ gboolean
 as_utils_search_token_valid (const gchar *token)
 {
 	guint i;
-	const gchar *blacklist[] = {
-		"and", "the", "application", "for", "you", "your",
-		"with", "can", "are", "from", "that", "use", "allows", "also",
-		"this", "other", "all", "using", "has", "some", "like", "them",
-		"well", "not", "using", "not", "but", "set", "its", "into",
-		"such", "was", "they", "where", "want", "only", "about",
-		"uses", "font", "features", "designed", "provides", "which",
-		"many", "used", "org", "fonts", "open", "more", "based",
-		"different", "including", "will", "multiple", "out", "have",
-		"each", "when", "need", "most", "both", "their", "even",
-		"way", "several", "been", "while", "very", "add", "under",
-		"what", "those", "much", "either", "currently", "one",
-		"support", "make", "over", "these", "there", "without", "etc",
-		"main",
-		NULL };
-	if (strlen (token) < 3)
-		return FALSE;
-	if (g_strstr_len (token, -1, "<") != NULL)
-		return FALSE;
-	if (g_strstr_len (token, -1, ">") != NULL)
-		return FALSE;
-	if (g_strstr_len (token, -1, "(") != NULL)
-		return FALSE;
-	if (g_strstr_len (token, -1, ")") != NULL)
-		return FALSE;
-	for (i = 0; blacklist[i] != NULL; i++)  {
-		if (g_strcmp0 (token, blacklist[i]) == 0)
+	for (i = 0; token[i] != '\0'; i++) {
+		if (token[i] == '<' ||
+		    token[i] == '>' ||
+		    token[i] == '(' ||
+		    token[i] == ')')
 			return FALSE;
 	}
+	if (i < 3)
+		return FALSE;
 	return TRUE;
 }
 
@@ -1896,3 +1879,69 @@ as_utils_unique_id_hash (const gchar *unique_id)
 	}
 	return hash;
 }
+
+static gboolean
+as_utils_appstream_id_is_valid_char (gchar ch)
+{
+	if (g_ascii_isalnum (ch))
+		return TRUE;
+	if (ch == '.')
+		return TRUE;
+	if (ch == '-')
+		return TRUE;
+	return FALSE;
+}
+
+/**
+ * as_utils_appstream_id_build:
+ * @str: a string to build the AppStream ID from
+ *
+ * Fixes a string to be a valid AppStream ID.
+ *
+ * This function replaces any invalid chars with an underscore.
+ *
+ * Returns: a valid AppStream ID, or %NULL if @str is invalid
+ *
+ * Since: 0.6.4
+ */
+gchar *
+as_utils_appstream_id_build (const gchar *str)
+{
+	gchar *tmp;
+	guint i;
+
+	/* invalid */
+	if (str == NULL)
+		return NULL;
+	if (str[0] == '\0')
+		return NULL;
+
+	tmp = g_strdup (str);
+	for (i = 0; tmp[i] != '\0'; i++) {
+		if (!as_utils_appstream_id_is_valid_char (tmp[i]))
+			tmp[i] = '_';
+	}
+	return tmp;
+}
+
+/**
+ * as_utils_appstream_id_valid:
+ * @str: a string
+ *
+ * Checks to see if a string is a valid AppStream ID. A valid AppStream ID only
+ * contains alpha-numeric chars, dots and dashes.
+ *
+ * Returns: %TRUE if the string is a valid AppStream ID
+ *
+ * Since: 0.6.4
+ */
+gboolean
+as_utils_appstream_id_valid (const gchar *str)
+{
+	guint i;
+	for (i = 0; str[i] != '\0'; i++) {
+		if (!as_utils_appstream_id_is_valid_char (str[i]))
+			return FALSE;
+	}
+	return TRUE;
+}
